"""
PyUnit Tests for the geostat.binning functions

TODO
Save arrays in another file?
Test direction and _in_bounds
"""

import unittest
import numpy as np
from numpy.testing import assert_array_almost_equal
from skgstat.binning import binify_even_width, binify_even_bin, group_to_bin, direction, _in_bounds

# Result arrays
result_binify_even_width_bm = np.array([[0., 3., 6., 9., 0., 3., 6., 9., 0., 3., 6., 9., 0.,
                                         3., 6., 9., 0., 3., 6., 9.],
                                        [3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6., 3.,
                                         0., 3., 6., 3., 0., 3., 6.],
                                        [6., 3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6.,
                                         3., 0., 3., 6., 3., 0., 3.],
                                        [9., 6., 3., 0., 9., 6., 3., 0., 9., 6., 3., 0., 9.,
                                         6., 3., 0., 9., 6., 3., 0.],
                                        [0., 3., 6., 9., 0., 3., 6., 9., 0., 3., 6., 9., 0.,
                                         3., 6., 9., 0., 3., 6., 9.],
                                        [3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6., 3.,
                                         0., 3., 6., 3., 0., 3., 6.],
                                        [6., 3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6.,
                                         3., 0., 3., 6., 3., 0., 3.],
                                        [9., 6., 3., 0., 9., 6., 3., 0., 9., 6., 3., 0., 9.,
                                         6., 3., 0., 9., 6., 3., 0.],
                                        [0., 3., 6., 9., 0., 3., 6., 9., 0., 3., 6., 9., 0.,
                                         3., 6., 9., 0., 3., 6., 9.],
                                        [3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6., 3.,
                                         0., 3., 6., 3., 0., 3., 6.],
                                        [6., 3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6.,
                                         3., 0., 3., 6., 3., 0., 3.],
                                        [9., 6., 3., 0., 9., 6., 3., 0., 9., 6., 3., 0., 9.,
                                         6., 3., 0., 9., 6., 3., 0.],
                                        [0., 3., 6., 9., 0., 3., 6., 9., 0., 3., 6., 9., 0.,
                                         3., 6., 9., 0., 3., 6., 9.],
                                        [3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6., 3.,
                                         0., 3., 6., 3., 0., 3., 6.],
                                        [6., 3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6.,
                                         3., 0., 3., 6., 3., 0., 3.],
                                        [9., 6., 3., 0., 9., 6., 3., 0., 9., 6., 3., 0., 9.,
                                         6., 3., 0., 9., 6., 3., 0.],
                                        [0., 3., 6., 9., 0., 3., 6., 9., 0., 3., 6., 9., 0.,
                                         3., 6., 9., 0., 3., 6., 9.],
                                        [3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6., 3.,
                                         0., 3., 6., 3., 0., 3., 6.],
                                        [6., 3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6.,
                                         3., 0., 3., 6., 3., 0., 3.],
                                        [9., 6., 3., 0., 9., 6., 3., 0., 9., 6., 3., 0., 9.,
                                         6., 3., 0., 9., 6., 3., 0.]])

result_binify_even_width_bw = np.array([0.42426407, 0.42426407, 0.42426407, 0.42426407, 0.42426407,
                                        0.42426407, 0.42426407, 0.42426407, 0.42426407, 0.42426407])

result_binify_even_width_bm_n6 = np.array([[0., 2., 4., 5., 0., 2., 4., 5., 0., 2., 4., 5., 0.,
                                            2., 4., 5., 0., 2., 4., 5.],
                                           [2., 0., 2., 4., 2., 0., 2., 4., 2., 0., 2., 4., 2.,
                                            0., 2., 4., 2., 0., 2., 4.],
                                           [4., 2., 0., 2., 4., 2., 0., 2., 4., 2., 0., 2., 4.,
                                            2., 0., 2., 4., 2., 0., 2.],
                                           [5., 4., 2., 0., 5., 4., 2., 0., 5., 4., 2., 0., 5.,
                                            4., 2., 0., 5., 4., 2., 0.],
                                           [0., 2., 4., 5., 0., 2., 4., 5., 0., 2., 4., 5., 0.,
                                            2., 4., 5., 0., 2., 4., 5.],
                                           [2., 0., 2., 4., 2., 0., 2., 4., 2., 0., 2., 4., 2.,
                                            0., 2., 4., 2., 0., 2., 4.],
                                           [4., 2., 0., 2., 4., 2., 0., 2., 4., 2., 0., 2., 4.,
                                            2., 0., 2., 4., 2., 0., 2.],
                                           [5., 4., 2., 0., 5., 4., 2., 0., 5., 4., 2., 0., 5.,
                                            4., 2., 0., 5., 4., 2., 0.],
                                           [0., 2., 4., 5., 0., 2., 4., 5., 0., 2., 4., 5., 0.,
                                            2., 4., 5., 0., 2., 4., 5.],
                                           [2., 0., 2., 4., 2., 0., 2., 4., 2., 0., 2., 4., 2.,
                                            0., 2., 4., 2., 0., 2., 4.],
                                           [4., 2., 0., 2., 4., 2., 0., 2., 4., 2., 0., 2., 4.,
                                            2., 0., 2., 4., 2., 0., 2.],
                                           [5., 4., 2., 0., 5., 4., 2., 0., 5., 4., 2., 0., 5.,
                                            4., 2., 0., 5., 4., 2., 0.],
                                           [0., 2., 4., 5., 0., 2., 4., 5., 0., 2., 4., 5., 0.,
                                            2., 4., 5., 0., 2., 4., 5.],
                                           [2., 0., 2., 4., 2., 0., 2., 4., 2., 0., 2., 4., 2.,
                                            0., 2., 4., 2., 0., 2., 4.],
                                           [4., 2., 0., 2., 4., 2., 0., 2., 4., 2., 0., 2., 4.,
                                            2., 0., 2., 4., 2., 0., 2.],
                                           [5., 4., 2., 0., 5., 4., 2., 0., 5., 4., 2., 0., 5.,
                                            4., 2., 0., 5., 4., 2., 0.],
                                           [0., 2., 4., 5., 0., 2., 4., 5., 0., 2., 4., 5., 0.,
                                            2., 4., 5., 0., 2., 4., 5.],
                                           [2., 0., 2., 4., 2., 0., 2., 4., 2., 0., 2., 4., 2.,
                                            0., 2., 4., 2., 0., 2., 4.],
                                           [4., 2., 0., 2., 4., 2., 0., 2., 4., 2., 0., 2., 4.,
                                            2., 0., 2., 4., 2., 0., 2.],
                                           [5., 4., 2., 0., 5., 4., 2., 0., 5., 4., 2., 0., 5.,
                                            4., 2., 0., 5., 4., 2., 0.]])

result_binify_even_width_bw_n6 = np.array([0.70710678, 0.70710678, 0.70710678, 0.70710678, 0.70710678,
                                           0.70710678])

binify_dm = np.array([[0., 1.41421356, 2.82842712, 4.24264069, 0.,
                       1.41421356, 2.82842712, 4.24264069, 0., 1.41421356,
                       2.82842712, 4.24264069, 0., 1.41421356, 2.82842712,
                       4.24264069, 0., 1.41421356, 2.82842712, 4.24264069],
                      [1.41421356, 0., 1.41421356, 2.82842712, 1.41421356,
                       0., 1.41421356, 2.82842712, 1.41421356, 0.,
                       1.41421356, 2.82842712, 1.41421356, 0., 1.41421356,
                       2.82842712, 1.41421356, 0., 1.41421356, 2.82842712],
                      [2.82842712, 1.41421356, 0., 1.41421356, 2.82842712,
                       1.41421356, 0., 1.41421356, 2.82842712, 1.41421356,
                       0., 1.41421356, 2.82842712, 1.41421356, 0.,
                       1.41421356, 2.82842712, 1.41421356, 0., 1.41421356],
                      [4.24264069, 2.82842712, 1.41421356, 0., 4.24264069,
                       2.82842712, 1.41421356, 0., 4.24264069, 2.82842712,
                       1.41421356, 0., 4.24264069, 2.82842712, 1.41421356,
                       0., 4.24264069, 2.82842712, 1.41421356, 0.],
                      [0., 1.41421356, 2.82842712, 4.24264069, 0.,
                       1.41421356, 2.82842712, 4.24264069, 0., 1.41421356,
                       2.82842712, 4.24264069, 0., 1.41421356, 2.82842712,
                       4.24264069, 0., 1.41421356, 2.82842712, 4.24264069],
                      [1.41421356, 0., 1.41421356, 2.82842712, 1.41421356,
                       0., 1.41421356, 2.82842712, 1.41421356, 0.,
                       1.41421356, 2.82842712, 1.41421356, 0., 1.41421356,
                       2.82842712, 1.41421356, 0., 1.41421356, 2.82842712],
                      [2.82842712, 1.41421356, 0., 1.41421356, 2.82842712,
                       1.41421356, 0., 1.41421356, 2.82842712, 1.41421356,
                       0., 1.41421356, 2.82842712, 1.41421356, 0.,
                       1.41421356, 2.82842712, 1.41421356, 0., 1.41421356],
                      [4.24264069, 2.82842712, 1.41421356, 0., 4.24264069,
                       2.82842712, 1.41421356, 0., 4.24264069, 2.82842712,
                       1.41421356, 0., 4.24264069, 2.82842712, 1.41421356,
                       0., 4.24264069, 2.82842712, 1.41421356, 0.],
                      [0., 1.41421356, 2.82842712, 4.24264069, 0.,
                       1.41421356, 2.82842712, 4.24264069, 0., 1.41421356,
                       2.82842712, 4.24264069, 0., 1.41421356, 2.82842712,
                       4.24264069, 0., 1.41421356, 2.82842712, 4.24264069],
                      [1.41421356, 0., 1.41421356, 2.82842712, 1.41421356,
                       0., 1.41421356, 2.82842712, 1.41421356, 0.,
                       1.41421356, 2.82842712, 1.41421356, 0., 1.41421356,
                       2.82842712, 1.41421356, 0., 1.41421356, 2.82842712],
                      [2.82842712, 1.41421356, 0., 1.41421356, 2.82842712,
                       1.41421356, 0., 1.41421356, 2.82842712, 1.41421356,
                       0., 1.41421356, 2.82842712, 1.41421356, 0.,
                       1.41421356, 2.82842712, 1.41421356, 0., 1.41421356],
                      [4.24264069, 2.82842712, 1.41421356, 0., 4.24264069,
                       2.82842712, 1.41421356, 0., 4.24264069, 2.82842712,
                       1.41421356, 0., 4.24264069, 2.82842712, 1.41421356,
                       0., 4.24264069, 2.82842712, 1.41421356, 0.],
                      [0., 1.41421356, 2.82842712, 4.24264069, 0.,
                       1.41421356, 2.82842712, 4.24264069, 0., 1.41421356,
                       2.82842712, 4.24264069, 0., 1.41421356, 2.82842712,
                       4.24264069, 0., 1.41421356, 2.82842712, 4.24264069],
                      [1.41421356, 0., 1.41421356, 2.82842712, 1.41421356,
                       0., 1.41421356, 2.82842712, 1.41421356, 0.,
                       1.41421356, 2.82842712, 1.41421356, 0., 1.41421356,
                       2.82842712, 1.41421356, 0., 1.41421356, 2.82842712],
                      [2.82842712, 1.41421356, 0., 1.41421356, 2.82842712,
                       1.41421356, 0., 1.41421356, 2.82842712, 1.41421356,
                       0., 1.41421356, 2.82842712, 1.41421356, 0.,
                       1.41421356, 2.82842712, 1.41421356, 0., 1.41421356],
                      [4.24264069, 2.82842712, 1.41421356, 0., 4.24264069,
                       2.82842712, 1.41421356, 0., 4.24264069, 2.82842712,
                       1.41421356, 0., 4.24264069, 2.82842712, 1.41421356,
                       0., 4.24264069, 2.82842712, 1.41421356, 0.],
                      [0., 1.41421356, 2.82842712, 4.24264069, 0.,
                       1.41421356, 2.82842712, 4.24264069, 0., 1.41421356,
                       2.82842712, 4.24264069, 0., 1.41421356, 2.82842712,
                       4.24264069, 0., 1.41421356, 2.82842712, 4.24264069],
                      [1.41421356, 0., 1.41421356, 2.82842712, 1.41421356,
                       0., 1.41421356, 2.82842712, 1.41421356, 0.,
                       1.41421356, 2.82842712, 1.41421356, 0., 1.41421356,
                       2.82842712, 1.41421356, 0., 1.41421356, 2.82842712],
                      [2.82842712, 1.41421356, 0., 1.41421356, 2.82842712,
                       1.41421356, 0., 1.41421356, 2.82842712, 1.41421356,
                       0., 1.41421356, 2.82842712, 1.41421356, 0.,
                       1.41421356, 2.82842712, 1.41421356, 0., 1.41421356],
                      [4.24264069, 2.82842712, 1.41421356, 0., 4.24264069,
                       2.82842712, 1.41421356, 0., 4.24264069, 2.82842712,
                       1.41421356, 0., 4.24264069, 2.82842712, 1.41421356,
                       0., 4.24264069, 2.82842712, 1.41421356, 0.]])

result_binify_even_width_bm_maxlag = np.array([[0., 3., 7., 9., 0., 3., 7., 9., 0., 3., 7., 9., 0.,
                                                3., 7., 9., 0., 3., 7., 9.],
                                               [3., 0., 3., 7., 3., 0., 3., 7., 3., 0., 3., 7., 3.,
                                                0., 3., 7., 3., 0., 3., 7.],
                                               [7., 3., 0., 3., 7., 3., 0., 3., 7., 3., 0., 3., 7.,
                                                3., 0., 3., 7., 3., 0., 3.],
                                               [9., 7., 3., 0., 9., 7., 3., 0., 9., 7., 3., 0., 9.,
                                                7., 3., 0., 9., 7., 3., 0.],
                                               [0., 3., 7., 9., 0., 3., 7., 9., 0., 3., 7., 9., 0.,
                                                3., 7., 9., 0., 3., 7., 9.],
                                               [3., 0., 3., 7., 3., 0., 3., 7., 3., 0., 3., 7., 3.,
                                                0., 3., 7., 3., 0., 3., 7.],
                                               [7., 3., 0., 3., 7., 3., 0., 3., 7., 3., 0., 3., 7.,
                                                3., 0., 3., 7., 3., 0., 3.],
                                               [9., 7., 3., 0., 9., 7., 3., 0., 9., 7., 3., 0., 9.,
                                                7., 3., 0., 9., 7., 3., 0.],
                                               [0., 3., 7., 9., 0., 3., 7., 9., 0., 3., 7., 9., 0.,
                                                3., 7., 9., 0., 3., 7., 9.],
                                               [3., 0., 3., 7., 3., 0., 3., 7., 3., 0., 3., 7., 3.,
                                                0., 3., 7., 3., 0., 3., 7.],
                                               [7., 3., 0., 3., 7., 3., 0., 3., 7., 3., 0., 3., 7.,
                                                3., 0., 3., 7., 3., 0., 3.],
                                               [9., 7., 3., 0., 9., 7., 3., 0., 9., 7., 3., 0., 9.,
                                                7., 3., 0., 9., 7., 3., 0.],
                                               [0., 3., 7., 9., 0., 3., 7., 9., 0., 3., 7., 9., 0.,
                                                3., 7., 9., 0., 3., 7., 9.],
                                               [3., 0., 3., 7., 3., 0., 3., 7., 3., 0., 3., 7., 3.,
                                                0., 3., 7., 3., 0., 3., 7.],
                                               [7., 3., 0., 3., 7., 3., 0., 3., 7., 3., 0., 3., 7.,
                                                3., 0., 3., 7., 3., 0., 3.],
                                               [9., 7., 3., 0., 9., 7., 3., 0., 9., 7., 3., 0., 9.,
                                                7., 3., 0., 9., 7., 3., 0.],
                                               [0., 3., 7., 9., 0., 3., 7., 9., 0., 3., 7., 9., 0.,
                                                3., 7., 9., 0., 3., 7., 9.],
                                               [3., 0., 3., 7., 3., 0., 3., 7., 3., 0., 3., 7., 3.,
                                                0., 3., 7., 3., 0., 3., 7.],
                                               [7., 3., 0., 3., 7., 3., 0., 3., 7., 3., 0., 3., 7.,
                                                3., 0., 3., 7., 3., 0., 3.],
                                               [9., 7., 3., 0., 9., 7., 3., 0., 9., 7., 3., 0., 9.,
                                                7., 3., 0., 9., 7., 3., 0.]])

result_binify_even_width_bw_maxlag = np.array([0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4])

result_binify_even_bin_bm = np.array([[0., 2., 6., 8., 0., 2., 6., 8., 0., 2., 6., 8., 0.,
                                       2., 6., 8., 0., 2., 6., 8.],
                                      [2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6., 2.,
                                       0., 2., 6., 2., 0., 2., 6.],
                                      [6., 2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6.,
                                       2., 0., 2., 6., 2., 0., 2.],
                                      [8., 6., 2., 0., 8., 6., 2., 0., 8., 6., 2., 0., 8.,
                                       6., 2., 0., 8., 6., 2., 0.],
                                      [0., 2., 6., 8., 0., 2., 6., 8., 0., 2., 6., 8., 0.,
                                       2., 6., 8., 0., 2., 6., 8.],
                                      [2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6., 2.,
                                       0., 2., 6., 2., 0., 2., 6.],
                                      [6., 2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6.,
                                       2., 0., 2., 6., 2., 0., 2.],
                                      [8., 6., 2., 0., 8., 6., 2., 0., 8., 6., 2., 0., 8.,
                                       6., 2., 0., 8., 6., 2., 0.],
                                      [0., 2., 6., 8., 0., 2., 6., 8., 0., 2., 6., 8., 0.,
                                       2., 6., 8., 0., 2., 6., 8.],
                                      [2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6., 2.,
                                       0., 2., 6., 2., 0., 2., 6.],
                                      [6., 2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6.,
                                       2., 0., 2., 6., 2., 0., 2.],
                                      [8., 6., 2., 0., 8., 6., 2., 0., 8., 6., 2., 0., 8.,
                                       6., 2., 0., 8., 6., 2., 0.],
                                      [0., 2., 6., 8., 0., 2., 6., 8., 0., 2., 6., 8., 0.,
                                       2., 6., 8., 0., 2., 6., 8.],
                                      [2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6., 2.,
                                       0., 2., 6., 2., 0., 2., 6.],
                                      [6., 2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6.,
                                       2., 0., 2., 6., 2., 0., 2.],
                                      [8., 6., 2., 0., 8., 6., 2., 0., 8., 6., 2., 0., 8.,
                                       6., 2., 0., 8., 6., 2., 0.],
                                      [0., 2., 6., 8., 0., 2., 6., 8., 0., 2., 6., 8., 0.,
                                       2., 6., 8., 0., 2., 6., 8.],
                                      [2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6., 2.,
                                       0., 2., 6., 2., 0., 2., 6.],
                                      [6., 2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6.,
                                       2., 0., 2., 6., 2., 0., 2.],
                                      [8., 6., 2., 0., 8., 6., 2., 0., 8., 6., 2., 0., 8.,
                                       6., 2., 0., 8., 6., 2., 0.]])

result_binify_even_bin_bw = np.array([0., 0., 1.41421356, 0., 0.,
                                      0., 1.41421356, 0., 1.41421356, 0.])

result_binify_even_bin_bm_n6 = np.array([[0., 1., 3., 5., 0., 1., 3., 5., 0., 1., 3., 5., 0.,
                                          1., 3., 5., 0., 1., 3., 5.],
                                         [1., 0., 1., 3., 1., 0., 1., 3., 1., 0., 1., 3., 1.,
                                          0., 1., 3., 1., 0., 1., 3.],
                                         [3., 1., 0., 1., 3., 1., 0., 1., 3., 1., 0., 1., 3.,
                                          1., 0., 1., 3., 1., 0., 1.],
                                         [5., 3., 1., 0., 5., 3., 1., 0., 5., 3., 1., 0., 5.,
                                          3., 1., 0., 5., 3., 1., 0.],
                                         [0., 1., 3., 5., 0., 1., 3., 5., 0., 1., 3., 5., 0.,
                                          1., 3., 5., 0., 1., 3., 5.],
                                         [1., 0., 1., 3., 1., 0., 1., 3., 1., 0., 1., 3., 1.,
                                          0., 1., 3., 1., 0., 1., 3.],
                                         [3., 1., 0., 1., 3., 1., 0., 1., 3., 1., 0., 1., 3.,
                                          1., 0., 1., 3., 1., 0., 1.],
                                         [5., 3., 1., 0., 5., 3., 1., 0., 5., 3., 1., 0., 5.,
                                          3., 1., 0., 5., 3., 1., 0.],
                                         [0., 1., 3., 5., 0., 1., 3., 5., 0., 1., 3., 5., 0.,
                                          1., 3., 5., 0., 1., 3., 5.],
                                         [1., 0., 1., 3., 1., 0., 1., 3., 1., 0., 1., 3., 1.,
                                          0., 1., 3., 1., 0., 1., 3.],
                                         [3., 1., 0., 1., 3., 1., 0., 1., 3., 1., 0., 1., 3.,
                                          1., 0., 1., 3., 1., 0., 1.],
                                         [5., 3., 1., 0., 5., 3., 1., 0., 5., 3., 1., 0., 5.,
                                          3., 1., 0., 5., 3., 1., 0.],
                                         [0., 1., 3., 5., 0., 1., 3., 5., 0., 1., 3., 5., 0.,
                                          1., 3., 5., 0., 1., 3., 5.],
                                         [1., 0., 1., 3., 1., 0., 1., 3., 1., 0., 1., 3., 1.,
                                          0., 1., 3., 1., 0., 1., 3.],
                                         [3., 1., 0., 1., 3., 1., 0., 1., 3., 1., 0., 1., 3.,
                                          1., 0., 1., 3., 1., 0., 1.],
                                         [5., 3., 1., 0., 5., 3., 1., 0., 5., 3., 1., 0., 5.,
                                          3., 1., 0., 5., 3., 1., 0.],
                                         [0., 1., 3., 5., 0., 1., 3., 5., 0., 1., 3., 5., 0.,
                                          1., 3., 5., 0., 1., 3., 5.],
                                         [1., 0., 1., 3., 1., 0., 1., 3., 1., 0., 1., 3., 1.,
                                          0., 1., 3., 1., 0., 1., 3.],
                                         [3., 1., 0., 1., 3., 1., 0., 1., 3., 1., 0., 1., 3.,
                                          1., 0., 1., 3., 1., 0., 1.],
                                         [5., 3., 1., 0., 5., 3., 1., 0., 5., 3., 1., 0., 5.,
                                          3., 1., 0., 5., 3., 1., 0.]])

result_binify_even_bin_bw_n6 = np.array([0., 1.41421356, 0., 1.41421356, 0.,
                                         1.41421356])

result_binify_even_bin_bm_maxlag = np.array([[0., 2., 6., 8., 0., 2., 6., 8., 0., 2., 6., 8., 0.,
                                              2., 6., 8., 0., 2., 6., 8.],
                                             [2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6., 2.,
                                              0., 2., 6., 2., 0., 2., 6.],
                                             [6., 2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6.,
                                              2., 0., 2., 6., 2., 0., 2.],
                                             [8., 6., 2., 0., 8., 6., 2., 0., 8., 6., 2., 0., 8.,
                                              6., 2., 0., 8., 6., 2., 0.],
                                             [0., 2., 6., 8., 0., 2., 6., 8., 0., 2., 6., 8., 0.,
                                              2., 6., 8., 0., 2., 6., 8.],
                                             [2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6., 2.,
                                              0., 2., 6., 2., 0., 2., 6.],
                                             [6., 2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6.,
                                              2., 0., 2., 6., 2., 0., 2.],
                                             [8., 6., 2., 0., 8., 6., 2., 0., 8., 6., 2., 0., 8.,
                                              6., 2., 0., 8., 6., 2., 0.],
                                             [0., 2., 6., 8., 0., 2., 6., 8., 0., 2., 6., 8., 0.,
                                              2., 6., 8., 0., 2., 6., 8.],
                                             [2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6., 2.,
                                              0., 2., 6., 2., 0., 2., 6.],
                                             [6., 2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6.,
                                              2., 0., 2., 6., 2., 0., 2.],
                                             [8., 6., 2., 0., 8., 6., 2., 0., 8., 6., 2., 0., 8.,
                                              6., 2., 0., 8., 6., 2., 0.],
                                             [0., 2., 6., 8., 0., 2., 6., 8., 0., 2., 6., 8., 0.,
                                              2., 6., 8., 0., 2., 6., 8.],
                                             [2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6., 2.,
                                              0., 2., 6., 2., 0., 2., 6.],
                                             [6., 2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6.,
                                              2., 0., 2., 6., 2., 0., 2.],
                                             [8., 6., 2., 0., 8., 6., 2., 0., 8., 6., 2., 0., 8.,
                                              6., 2., 0., 8., 6., 2., 0.],
                                             [0., 2., 6., 8., 0., 2., 6., 8., 0., 2., 6., 8., 0.,
                                              2., 6., 8., 0., 2., 6., 8.],
                                             [2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6., 2.,
                                              0., 2., 6., 2., 0., 2., 6.],
                                             [6., 2., 0., 2., 6., 2., 0., 2., 6., 2., 0., 2., 6.,
                                              2., 0., 2., 6., 2., 0., 2.],
                                             [8., 6., 2., 0., 8., 6., 2., 0., 8., 6., 2., 0., 8.,
                                              6., 2., 0., 8., 6., 2., 0.]])

result_binify_even_bin_bw_maxlag = np.array([0., 0., 1.41421356, 0., 0.,
                                             0., 1.41421356, 0., 1.41421356, 0.])

result_group_to_bin = [
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3,
     3, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3,
     3, 3, 3, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3,
     3, 3, 3, 3, 3, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3,
     3, 3, 3, 3, 3, 3, 3, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
     3, 3, 3, 3, 3, 3, 3, 3, 3, 3],
    [],
    [],
    [0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 2, 1, 2, 3, 2, 1, 2, 3,
     2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 3, 2, 3, 2, 3, 2, 3, 2, 3, 2, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 2, 1, 0,
     1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 3, 2, 3, 2,
     3, 2, 3, 2, 3, 2, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 2, 1,
     2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 3, 2, 3, 2, 3, 2, 3, 2, 3, 2, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1,
     1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1,
     2, 3, 3, 2, 3, 2, 3, 2, 3, 2, 3, 2, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2,
     1, 0, 1, 2, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 3, 2, 3, 2, 3, 2, 3, 2, 3, 2],
    [],
    [],
    [0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 3, 1, 3, 1, 3, 1, 3, 1,
     3, 1, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 3, 1, 3, 1, 3, 1,
     3, 1, 3, 1, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 3, 1, 3, 1,
     3, 1, 3, 1, 3, 1, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 3, 1,
     3, 1, 3, 1, 3, 1, 3, 1, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0,
     3, 1, 3, 1, 3, 1, 3, 1, 3, 1],
    [],
    [],
    [0, 3, 0, 3, 0, 3, 0, 3, 0, 3, 3, 0, 3, 0, 3, 0, 3, 0, 3, 0, 0, 3, 0, 3, 0, 3, 0, 3, 0, 3, 3, 0, 3, 0, 3, 0, 3, 0,
     3, 0, 0, 3, 0, 3, 0, 3, 0, 3, 0, 3, 3, 0, 3, 0, 3, 0, 3, 0, 3, 0, 0, 3, 0, 3, 0, 3, 0, 3, 0, 3, 3, 0, 3, 0, 3, 0,
     3, 0, 3, 0, 0, 3, 0, 3, 0, 3, 0, 3, 0, 3, 3, 0, 3, 0, 3, 0, 3, 0, 3, 0]]

group_to_bin_bm = np.array([[0., 3., 6., 9., 0., 3., 6., 9., 0., 3., 6., 9., 0.,
                             3., 6., 9., 0., 3., 6., 9.],
                            [3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6., 3.,
                             0., 3., 6., 3., 0., 3., 6.],
                            [6., 3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6.,
                             3., 0., 3., 6., 3., 0., 3.],
                            [9., 6., 3., 0., 9., 6., 3., 0., 9., 6., 3., 0., 9.,
                             6., 3., 0., 9., 6., 3., 0.],
                            [0., 3., 6., 9., 0., 3., 6., 9., 0., 3., 6., 9., 0.,
                             3., 6., 9., 0., 3., 6., 9.],
                            [3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6., 3.,
                             0., 3., 6., 3., 0., 3., 6.],
                            [6., 3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6.,
                             3., 0., 3., 6., 3., 0., 3.],
                            [9., 6., 3., 0., 9., 6., 3., 0., 9., 6., 3., 0., 9.,
                             6., 3., 0., 9., 6., 3., 0.],
                            [0., 3., 6., 9., 0., 3., 6., 9., 0., 3., 6., 9., 0.,
                             3., 6., 9., 0., 3., 6., 9.],
                            [3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6., 3.,
                             0., 3., 6., 3., 0., 3., 6.],
                            [6., 3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6.,
                             3., 0., 3., 6., 3., 0., 3.],
                            [9., 6., 3., 0., 9., 6., 3., 0., 9., 6., 3., 0., 9.,
                             6., 3., 0., 9., 6., 3., 0.],
                            [0., 3., 6., 9., 0., 3., 6., 9., 0., 3., 6., 9., 0.,
                             3., 6., 9., 0., 3., 6., 9.],
                            [3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6., 3.,
                             0., 3., 6., 3., 0., 3., 6.],
                            [6., 3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6.,
                             3., 0., 3., 6., 3., 0., 3.],
                            [9., 6., 3., 0., 9., 6., 3., 0., 9., 6., 3., 0., 9.,
                             6., 3., 0., 9., 6., 3., 0.],
                            [0., 3., 6., 9., 0., 3., 6., 9., 0., 3., 6., 9., 0.,
                             3., 6., 9., 0., 3., 6., 9.],
                            [3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6., 3.,
                             0., 3., 6., 3., 0., 3., 6.],
                            [6., 3., 0., 3., 6., 3., 0., 3., 6., 3., 0., 3., 6.,
                             3., 0., 3., 6., 3., 0., 3.],
                            [9., 6., 3., 0., 9., 6., 3., 0., 9., 6., 3., 0., 9.,
                             6., 3., 0., 9., 6., 3., 0.]])

result_group_to_bin_maxlag = [
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3,
     3, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3,
     3, 3, 3, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3,
     3, 3, 3, 3, 3, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3,
     3, 3, 3, 3, 3, 3, 3, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
     3, 3, 3, 3, 3, 3, 3, 3, 3, 3], [], [],
    [0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 2, 1, 2, 3, 2, 1, 2, 3,
     2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 3, 2, 3, 2, 3, 2, 3, 2, 3, 2, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 2, 1, 0,
     1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 3, 2, 3, 2,
     3, 2, 3, 2, 3, 2, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 2, 1,
     2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 3, 2, 3, 2, 3, 2, 3, 2, 3, 2, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1,
     1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1,
     2, 3, 3, 2, 3, 2, 3, 2, 3, 2, 3, 2, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2,
     1, 0, 1, 2, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 2, 1, 2, 3, 3, 2, 3, 2, 3, 2, 3, 2, 3, 2], [], [], [],
    [0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 3, 1, 3, 1, 3, 1, 3, 1,
     3, 1, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 3, 1, 3, 1, 3, 1,
     3, 1, 3, 1, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 3, 1, 3, 1,
     3, 1, 3, 1, 3, 1, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 3, 1,
     3, 1, 3, 1, 3, 1, 3, 1, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0,
     3, 1, 3, 1, 3, 1, 3, 1, 3, 1], [],
    [0, 3, 0, 3, 0, 3, 0, 3, 0, 3, 3, 0, 3, 0, 3, 0, 3, 0, 3, 0, 0, 3, 0, 3, 0, 3, 0, 3, 0, 3, 3, 0, 3, 0, 3, 0, 3, 0,
     3, 0, 0, 3, 0, 3, 0, 3, 0, 3, 0, 3, 3, 0, 3, 0, 3, 0, 3, 0, 3, 0, 0, 3, 0, 3, 0, 3, 0, 3, 0, 3, 3, 0, 3, 0, 3, 0,
     3, 0, 3, 0, 0, 3, 0, 3, 0, 3, 0, 3, 0, 3, 3, 0, 3, 0, 3, 0, 3, 0, 3, 0]]

result_group_to_bin_azimuth_and_tolerance = [
    [0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 0, 0, 0, 0, 0, 0,
     0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1,
     1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2,
     2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2,
     3, 3, 3, 3, 3, 3, 3, 3], [], [], [], [], [], [], [], [], []]


class TestBinning(unittest.TestCase):
    def setUp(self):
        self.coordinates = np.array([(0, 0), (1, 1), (2, 2), (3, 3)] * 5)
        self.values = np.array([0, 1, 2, 3] * 5)

    def test_binify_even_width_bm(self):
        """
        Testing binning matrix of binify_even_width Function
        """

        assert_array_almost_equal(np.asarray(binify_even_width(self.coordinates)[0]),
                                  result_binify_even_width_bm)

    def test_binify_even_width_bw(self):
        """
        Testing binning width array of binify_even_width Function
        """

        assert_array_almost_equal(np.asarray(binify_even_width(self.coordinates)[1]),
                                  result_binify_even_width_bw)

    def test_binify_even_width_bm_n6(self):
        """
        Testing binning matrix of binify_even_width Function with 6 bins
        """

        assert_array_almost_equal(np.asarray(binify_even_width(self.coordinates, N=6)[0]),
                                  result_binify_even_width_bm_n6)

    def test_binify_even_width_bw_n6(self):
        """
        Testing binning width array of binify_even_width Function with 6 bins
        """

        assert_array_almost_equal(np.asarray(binify_even_width(self.coordinates, N=6)[1]),
                                  result_binify_even_width_bw_n6)


    def test_binify_even_width_bm_with_dm(self):
        """
        Testing binning matrix of binify_even_width Function with a distance matrix
        """

        assert_array_almost_equal(np.asarray(binify_even_width(self.coordinates, dm=binify_dm)[0]),
                                  result_binify_even_width_bm)

    def test_binify_even_width_bm_maxlag(self):
        """
        Testing binning matrix of binify_even_width Function with a maxlag
        """

        assert_array_almost_equal(np.asarray(binify_even_width(self.coordinates, maxlag=4)[0]),
                                  result_binify_even_width_bm_maxlag)

    def test_binify_even_width_bw_maxlag(self):
        """
        Testing binning matrix of binify_even_width Function with a maxlag
        """

        assert_array_almost_equal(np.asarray(binify_even_width(self.coordinates, maxlag=4)[1]),
                                  result_binify_even_width_bw_maxlag)

    def test_binify_even_bin_bm(self):
        """
        Testing binning matrix of binify_even_bin Function
        """

        assert_array_almost_equal(np.asarray(binify_even_bin(self.coordinates)[0]),
                                  result_binify_even_bin_bm)

    def test_binify_even_bin_bw(self):
        """
        Testing binning width array of binify_even_bin Function
        """

        assert_array_almost_equal(np.asarray(binify_even_bin(self.coordinates)[1]),
                                  result_binify_even_bin_bw)

    def test_binify_even_bin_bm_n6(self):
        """
        Testing binning matrix of binify_even_bin Function with 6 bins
        """

        assert_array_almost_equal(np.asarray(binify_even_bin(self.coordinates, N=6)[0]),
                                  result_binify_even_bin_bm_n6)

    def test_binify_even_bin_bw_n6(self):
        """
        Testing binning width array of binify_even_bin Function with 6 bins
        """

        assert_array_almost_equal(np.asarray(binify_even_bin(self.coordinates, N=6)[1]),
                                  result_binify_even_bin_bw_n6)

    def test_binify_even_bin_bm_maxlag(self):
        """
        Testing binning matrix of binify_even_bin Function with a maxlag
        """

        assert_array_almost_equal(np.asarray(binify_even_bin(self.coordinates, maxlag=4)[0]),
                                  result_binify_even_bin_bm_maxlag)

    def test_binify_even_bin_bw_maxlag(self):
        """
        Testing binning width array of binify_even_bin Function with a maxlag
        """

        assert_array_almost_equal(np.asarray(binify_even_bin(self.coordinates, maxlag=4)[1]),
                                  result_binify_even_bin_bw_maxlag)

    def test_binify_group_to_bin(self):
        """
        Testing group_to_bin function
        """

        self.assertEqual(group_to_bin(self.values, X=self.coordinates), result_group_to_bin)

    def test_binify_group_to_bin_with_bm(self):
        """
        Testing group_to_bin function
        """

        self.assertEqual(group_to_bin(self.values, bm=group_to_bin_bm), result_group_to_bin)

    def test_binify_group_to_bin_with_bm(self):
        """
        Testing group_to_bin function
        """

        self.assertEqual(group_to_bin(self.values, bm=group_to_bin_bm), result_group_to_bin)

    def test_binify_group_to_bin_with_maxlag(self):
        """
        Testing group_to_bin function with maxlag
        """

        self.assertEqual(group_to_bin(self.values, X=self.coordinates, maxlag=4), result_group_to_bin_maxlag)

    def test_binify_group_to_bin_with_azimuth_and_tolerance(self):
        """
        Testing group_to_bin function with azimuth and tolerance
        """

        self.assertEqual(group_to_bin(self.values, X=self.coordinates, azimuth_deg=90, tolerance=22.5),
                         result_group_to_bin_azimuth_and_tolerance)

    def test_in_bounds(self):
        """        Testing in_bounds function with result = True
        """

        self.assertTrue(_in_bounds(50, 40, 10))
        self.assertFalse(_in_bounds(50, 10, 40))



if __name__ == '__main__':
    unittest.main()
